/*=========================================================================

  Program:   wxMaracas
  Module:    $RCSfile: wxMaracasMultipleVolumeRendererManager.h,v $
  Language:  C++
  Date:      $Date: 2009/11/20 17:09:05 $
  Version:   $Revision: 1.7 $

  Copyright: (c) 2002, 2003
  License:
  
     This software is distributed WITHOUT ANY WARRANTY; without even 
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
     PURPOSE.  See the above copyright notice for more information.

=========================================================================*/




#ifndef __wxMaracasMultipleVolumeRendererManagerH__
#define __wxMaracasMultipleVolumeRendererManagerH__

#include <iostream>
#include <vector>
#include <vtkMatrix4x4.h>

#include "wxMaracasMultipleVolumeRendererManagerData.h"


class wxMaracasMultipleVolumeRendererManager  {

public:
	wxMaracasMultipleVolumeRendererManager();
	~wxMaracasMultipleVolumeRendererManager();

	/**
	**	Sets the renderer to manage the prop3D from the view
	**/
	void setRenderer(vtkRenderer*  renderer);

	/**
	**	Gets the renderer to manage the prop3D from the view
	**/
	vtkRenderer* getRenderer();
	
	/**
	**	Adds a volume
	**/
	int addVolume(int idTP, vtkImageData* img, std::string dataname) throw (char*);	
	
	/**
	**	loads a prop3D from a nSTL file
	**/
	vtkProp3D* getVolume(std::string filename);

	/**
	**	loads a MHD file to convert it into an actor
	**/
	vtkImageData* getImageData(std::string filename);

	/**
	** Gets image data asotiated with the rendering manager
	**/
	vtkImageData* getImageData();

	/**
	**	adds or removes an actor depending of the bool value
	**/
	void addRemoveActor(int propid, bool addremove) throw(char*);
	
	/**
	**	Check if the variables are setted correctly
	**/
	void checkInvariant()throw(char*);

	/** 
	** Set Volume Opacity
	**/
	void setVolumeOpacity(int propid, std::vector<double> greylevel,std::vector<double> value) throw(char*);
	/**
	**	Set Volume Color
	**/
	void setVolumeColor(int volid, std::vector<double> greylevel,
									std::vector<double> red,
									std::vector<double> green,
									std::vector<double> blue)throw(char*);	

	/**
	** Given an id search the data in the vector
	**/
	wxMaracasMultipleVolumeRendererManagerData* getViewData(int id)throw(char*);

	void changeCompositeMIPFunction(int id, int function) throw (char *);
	
	/**
	** Deletes given actor
	**/
	void deleteActor(int volumeid)throw (char *);

	/**
	** Updates given volume
	**/
	void Update(int propid);
	
	vtkPiecewiseFunction* GetTransferFunction(int volumeid);
	vtkColorTransferFunction* GetColorFunction(int volumeid);
private:	
	std::vector<wxMaracasMultipleVolumeRendererManagerData*> prop3Dvect;

	vtkRenderer*  _renderer;
	vtkImageData* image;

	int _idCount;

	

	
};

#endif 
