
#include "wxMaracasSurfaceRenderingManagerDataMhd.h"

#include "vtkStripper.h"

wxMaracasSurfaceRenderingManagerDataMhd::wxMaracasSurfaceRenderingManagerDataMhd(vtkImageData* imagedata, std::string dataname, vtkRenderWindowInteractor*  interactor)
: wxMaracasSurfaceRenderingManagerData(NULL, dataname)
{

	this->setVtkImageData(imagedata);
	_dataname = dataname;
	
	_maxgreylevel = getMaxLevel(imagedata);
	_prop3D=NULL;			

	_cubesFilter = vtkMarchingCubes::New();	
	_cubesFilter->SetInput(this->_imagedata);		
	_cubesFilter->ComputeGradientsOn ();
	_cubesFilter->ComputeScalarsOn ();
	_cubesFilter->SetNumberOfContours( 1 );
	_cleanFilter = vtkCleanPolyData::New();		
	_cleanFilter->SetInput ( _cubesFilter->GetOutput() );

	_dataMapper = vtkPolyDataMapper::New( );
	_dataMapper->ScalarVisibilityOff( );
	_dataMapper->ImmediateModeRenderingOn();
	vtkActor* dataActor = vtkActor::New();

	//if(_boxWidgetS1){
	if(interactor){

		_boxWidgetS1 = vtkBoxWidget::New();
		_boxWidgetS1->SetInteractor( interactor );
		_boxWidgetS1->SetPlaceFactor(1.25); 

		_boxWidgetS1->SetInput( this->_imagedata );
		_boxWidgetS1->PlaceWidget();	
		boxSurfaceObserver* observer = boxSurfaceObserver::New();
		
		_boxWidgetS1->HandlesOn ();
		_boxWidgetS1->On();

		vtkStripper* striper = vtkStripper::New();
	    striper->SetInput( _cleanFilter->GetOutput() );
		//striper->SetInput( _cubesFilter->GetOutput() );
		
		striper->Update();
		_boxWidgetS1->SetInput(striper->GetOutput());
		_boxWidgetS1->PlaceWidget();

		_boxWidgetS1->HandlesOn ();
		_boxWidgetS1->On();

		_tissuePlanes  = vtkPlanes::New();
   
		int x1,x2,y1,y2,z1,z2;
		this->_imagedata->GetExtent(x1,x2,y1,y2,z1,z2);
		_tissuePlanes->SetBounds  (x1,x2,y1,y2,z1,z2);


		
		_boxWidgetS1->GetPlanes( _tissuePlanes );
		
		_tissueClipper = vtkClipPolyData::New();
		_tissueClipper->SetInput( striper->GetOutput() );
		_tissueClipper->SetClipFunction( _tissuePlanes );
		_tissueClipper->InsideOutOn( );
		_dataMapper->SetInput( _tissueClipper->GetOutput() );
		observer->SetPlanes( _tissuePlanes );
		observer->SetActor( dataActor );
		_boxWidgetS1->AddObserver( vtkCommand::InteractionEvent		 , observer );


		
	}else{		
		_dataMapper->SetInput(_cleanFilter->GetOutput());				
	}
	
	dataActor->SetMapper(_dataMapper);	
	this->_prop3D = dataActor;

	
	

	this->changeIsoValue(this->_maxgreylevel);	


}


wxMaracasSurfaceRenderingManagerDataMhd::~wxMaracasSurfaceRenderingManagerDataMhd()
{
	_cubesFilter->Delete();
	_cleanFilter->Delete();
	_dataMapper->Delete();
}
	
void wxMaracasSurfaceRenderingManagerDataMhd::UpdateSurface()
{
	_cubesFilter->Update();    
	_cleanFilter->Update();
	_dataMapper->Update();	
}
/**
**	changes the isovalue in a prop3D
**/
void wxMaracasSurfaceRenderingManagerDataMhd::changeIsoValue(double value){	
		
   
    _cubesFilter->SetValue(0,value);		
	_cubesFilter->Update();    
	_cleanFilter->Update();
	_dataMapper->Update();	
	
	
	
}
int wxMaracasSurfaceRenderingManagerDataMhd::getMaxGreyLevel(){
	return _maxgreylevel;
}

/**
	** Get's the max grey level of the image
	**/
int wxMaracasSurfaceRenderingManagerDataMhd::getMaxLevel(vtkImageData* img){

	int ext[6], i, j, k,max=0;
	img->GetExtent(ext);

	for(i = ext[0]; i < ext[1];i++){
		for(j = ext[2]; j < ext[3];j++){
			for(k = ext[4]; k < ext[5];k++){
                unsigned short* ptr = (unsigned short*)img->GetScalarPointer(i,j,k);
				int temp = (int)*ptr;
				if(temp > max){
                    max = temp;
				}
			}
		}
	}
	return max;


}

/**
**	Sets the VTK image data
**/

void wxMaracasSurfaceRenderingManagerDataMhd::setVtkImageData(vtkImageData* imagedata){
	_imagedata = imagedata;
}

